# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2014 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from .slaprunner import SlaprunnerTestSuite

import json
import random
import string
import time
import requests

class NotHttpOkException(Exception):
  pass

class GitlabTestSuite(SlaprunnerTestSuite):
  """
  Run Gitlab inside Slaprunner Resiliency Test.
  Note: requires specific kernel allowing long shebang paths.
  """

  def _setGitlabInstanceParameter(self):
    """
    Set inside of slaprunner the instance parameter to use to deploy gitlab instance.
    """
    self.logger.info('Updating instance parameter to use to deploy gitlab instance...')
    self._connectToSlaprunner(
        resource='saveParameterXml',
        data='software_type=gitlab-test&parameter=%3C%3Fxml%20version%3D%221.0%22%20encoding%3D%22utf-8%22%3F%3E%0A%3Cinstance%3E%0A%3C%2Finstance%3E'
    )

  def _connectToGitlab(self, path='', post_data=None, url='', parameter_dict={}):
    request_url = self.backend_url
    if url:
      request_url = url
    if path:
      request_url += '/' + path
    headers = {"PRIVATE-TOKEN" : self.private_token}
    if post_data is None:
      response = requests.get(request_url, params=parameter_dict,
                              headers=headers, verify=False)
    elif post_data == {}:
      response = requests.post(request_url, params=parameter_dict,
                                headers=headers, verify=False)
    else:
      response = requests.post(request_url, params=parameter_dict,
                                headers=headers, data=post_data, verify=False)
    if not response.ok:
      raise Exception("ERROR connecting to Gitlab: %s: %s \n%s" % (
        response.status_code, response.reason, response.text))
    return response.text

  def _createNewProject(self, name, namespace='open'):
    uri = 'api/v3/projects'
    parameter_dict = {'name': name, 'namespace': namespace}
    return self._connectToGitlab(uri, post_data={}, parameter_dict=parameter_dict)

  def _listProjects(self):
    path = 'api/v3/projects'
    return json.loads(self._connectToGitlab(path=path))

  def _setGitlabConnectionParameter(self):
    """
    set parameters of gitlab instance.
    Note: it is not connection parameter of slaprunner,
    but connection parameter of what is inside of webrunner.
    """
    data = self._connectToSlaprunner(
        resource='getConnectionParameter/slappart0'
    )
    parameter_dict = json.loads(data)
    self.backend_url = parameter_dict['backend_url']
    self.password = parameter_dict['password']
    self.private_token = parameter_dict['private-token']
    self.file_uri = parameter_dict['latest-file-uri']

  def _getRootPassword(self):
    data = self._connectToSlaprunner(
        resource='getConnectionParameter/slappart0'
    )
    password = json.loads(json.loads(data)['_'])['password']
    self.logger.info('Retrieved gitlab root password is:\n%s' % password)
    return password

  def generateData(self):
    self.slaprunner_password = ''.join(
        random.SystemRandom().sample(string.ascii_lowercase, 8)
    )
    self.slaprunner_user = 'slapos'
    self.logger.info('Generated slaprunner user is: %s' % self.slaprunner_user)
    self.logger.info('Generated slaprunner password is: %s' % self.slaprunner_password)

  def pushDataOnMainInstance(self):
    """
    Create a dummy Software Release,
    Build it,
    Wait for build to be successful,
    Deploy instance,
    Wait for instance to be started.
    Store the main IP of the slaprunner for future use.
    """

    self.logger.debug('Getting the backend URL...')
    parameter_dict = self._getPartitionParameterDict()
    self.slaprunner_backend_url = parameter_dict['backend-url']
    self.logger.info('backend_url is %s.' % self.slaprunner_backend_url)
    self.slaprunner_user = parameter_dict['init-user']
    self.slaprunner_password = parameter_dict['init-password']

    self._login()
    time.sleep(10)

    self._gitClone()
    self._openSoftwareRelease('gitlab')
    self._setGitlabInstanceParameter()

    self.logger.info('Waiting for 1 minutes...')
    # Debug, remove me.
    time.sleep(60)

    self._buildSoftwareRelease()
    self._deployInstance()
    # wait for unicorn to start then run instance again to install gitlab backup
    self.logger.info('wait 1 minute for unicorn to start then run instance...')
    time.sleep(60)
    self._deployInstance()
    self._deployInstance()
    # Stop all services because we want to restart gitlab (safe)
    self.logger.info('Stop all services because we want to restart gitlab..')
    self._connectToSlaprunner('/stopAllPartition')
    self._deployInstance()

    self._setGitlabConnectionParameter()
    self.logger.info('Retrieved gitlab url is:\n%s' % self.backend_url)
    self.logger.info('Gitlab root password is:\n%s' % self.password)
    self.logger.info('Gitlab private token is:\n%s' % self.private_token)

    self.logger.info('Waiting 1 minute so that gitlab can be started...')
    time.sleep(60)

    print self._createNewProject('sample.test')
    project_list = self._listProjects()
    self.default_project_list = []
    for project in project_list:
      self.default_project_list.append(project['name_with_namespace'])

    self.logger.info('Gitlab project list is:\n%s' % self.default_project_list)
    self.logger.info('Getting test file at url: %s' % self.file_uri)
    self.sample_file = self._connectToGitlab(url=self.file_uri)

    self.logger.info('Wait 10 minutes for main instance to have backup of gitlab...')
    time.sleep(60) #0)

    # in erp5testnode, we have only one IP, so we can't run at the same time
    # gitlab in webrunner of main instance, and other services in import script of clone instance
    # So we stop main instance processes.
    self._connectToSlaprunner('/stopAllPartition')

    self.logger.info('Wait half an hour for clone to have compiled Gitlab SR...')
    #time.sleep(3600 / 2)
    time.sleep(600)

  def checkDataOnCloneInstance(self):
    """
    Check that:
      * backend_url is different
      * Software Release profile is the same,
      * Software Release is built and is the same,
      * Instance is deployed and is the same (contains same new data).
    """
    old_slaprunner_backend_url = self.slaprunner_backend_url
    self.slaprunner_backend_url = self._returnNewInstanceParameter(
        parameter_key='backend-url',
        old_parameter_value=old_slaprunner_backend_url,
        force_new=True,
    )
    self._login()
    self._waitForSoftwareBuild()
    self._deployInstance()
    time.sleep(60)

    project_list = self._listProjects()
    success = True
    for project in project_list:
      success = success and (project['name_with_namespace'] in self.default_project_list)

    if success:
      file_content = self._connectToGitlab(url=self.file_uri)
      success = success and (file_content == self.sample_file)

    if success:
      self.logger.info('Data are the same: success.')
      return True
    else:
      self.logger.info('Data are different: failure.')
      return False


def runTestSuite(*args, **kwargs):
  """
  Run Slaprunner Resiliency Test.
  """
  return GitlabTestSuite(*args, **kwargs).runTestSuite()




#!/usr/bin/env python

import sys
import os
import glob
import json
import ConfigParser
import time
from datetime import datetime
import base64
import hashlib
import PyRSS2Gen

def getKey(item):
  return item.pubDate

class monitorFeed(object):

  def __init__(self, instance_name, hosting_name,
      public_url, private_url, feed_url):
    self.rss_item_list = []
    self.report_date = datetime.utcnow()
    self.instance_name = instance_name
    self.hosting_name = hosting_name
    self.public_url = public_url
    self.private_url = private_url
    self.feed_url = feed_url

  def appendItem(self, item_dict):
    event_time = datetime.fromtimestamp(item_dict['change-time'])
    description = item_dict.get('message', '')
    rss_item = PyRSS2Gen.RSSItem(
      categories = [item_dict['status']],
      source = PyRSS2Gen.Source(item_dict['title'], self.public_url),
      title = '[%s] %s' % (item_dict['status'], item_dict['title']),
      comments = description,
      description = "%s: %s\n%s" % (event_time, item_dict['status'], description),
      link = self.private_url,
      pubDate = event_time,
      guid = PyRSS2Gen.Guid(base64.b64encode("%s, %s" % (self.hosting_name,
                                                         item_dict['title'])))
    )
    self.rss_item_list.append(rss_item)

  def genrss(self, output_file):
    ### Build the rss feed
    sorted(self.rss_item_list, key=getKey)
    rss_feed = PyRSS2Gen.RSS2 (
      title = self.instance_name,
      link = self.feed_url,
      description = self.hosting_name,
      lastBuildDate = self.report_date,
      items = self.rss_item_list
    )
  
    with open(output_file, 'w') as frss:
      frss.write(rss_feed.to_xml())

def softConfigGet(config, *args, **kwargs):
  try:
    return config.get(*args, **kwargs)
  except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    return ""

def generateStatisticsData(stat_file_path, content):
  # csv document for success/error statictics
  if not os.path.exists(stat_file_path) or os.stat(stat_file_path).st_size == 0:
    with open(stat_file_path, 'w') as fstat:
      data_dict = {
        "date": time.time(),
        "data": ["Date, Success, Error, Warning"]
      }
      fstat.write(json.dumps(data_dict))

  current_state = ''
  if content.has_key('state'):
    current_state = '%s, %s, %s, %s' % (
      content['date'],
      content['state']['success'],
      content['state']['error'],
      content['state']['warning'])

  # append to file
  if current_state:
    with open (stat_file_path, mode="r+") as fstat:
      fstat.seek(0,2)
      position = fstat.tell() -2
      fstat.seek(position)
      fstat.write('%s}' % ',"{}"]'.format(current_state))

def run(args_list):
  monitor_file, instance_file = args_list

  monitor_config = ConfigParser.ConfigParser()
  monitor_config.read(monitor_file)

  base_folder = monitor_config.get('monitor', 'private-folder')
  status_folder = monitor_config.get('monitor', 'public-folder')
  base_url = monitor_config.get('monitor', 'base-url')
  related_monitor_list = monitor_config.get("monitor", "monitor-url-list").split()
  statistic_folder = os.path.join(base_folder, 'data', '.jio_documents')
  parameter_file = os.path.join(base_folder, 'config', '.jio_documents', 'config.json')
  feed_output = os.path.join(status_folder, 'feed')
  public_url = "%s/share/jio_public/" % base_url
  private_url = "%s/share/jio_private/" % base_url
  feed_url = "%s/public/feed" % base_url

  error = warning = success = 0
  status = 'OK'
  promise_list = []
  global_state_file = os.path.join(base_folder, 'monitor.global.json')
  public_state_file = os.path.join(status_folder, 'monitor.global.json')

  if not os.path.exists(statistic_folder):
    try:
      os.makedirs(statistic_folder)
    except OSError, e:
      if e.errno == os.errno.EEXIST and os.path.isdir(statistic_folder):
        pass
      else: raise

  # search for all status files
  file_list = filter(os.path.isfile,
      glob.glob("%s/*.status.json" % status_folder)
    )
  if os.path.exists(instance_file):
    config = ConfigParser.ConfigParser()
    config.read(instance_file)
  else:
    raise Exception("Cannot read instance configuration at %s" % instance_file)

  report_date = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
  monitor_feed = monitorFeed(
    config.get('instance', 'name'),
    config.get('instance', 'root-name'),
    public_url,
    private_url,
    feed_url)
  for file in file_list:
    try:
      with open(file, 'r') as temp_file:
        tmp_json = json.loads(temp_file.read())
    except ValueError:
      # bad json file ?
      continue
    if tmp_json['status'] == 'ERROR':
      error  += 1
    elif tmp_json['status'] == 'OK':
      success += 1
    elif tmp_json['status'] == 'WARNING':
      warning += 1
    tmp_json['time'] = tmp_json['start-date'].split(' ')[1]
    promise_list.append(tmp_json)
    monitor_feed.appendItem(tmp_json)

  if error:
    status = 'ERROR'
  elif warning:
    status = 'WARNING'

  monitor_feed.genrss(feed_output)
  global_state_dict = dict(
    
    status=status,
    state={
      'error': error,
      'success': success,
      'warning': warning,
    },
    type='global',
    date=report_date,
    _links={"rss_url": {"href": "%s/public/feed" % base_url},
            "public_url": {"href": public_url},
            "private_url": {"href": private_url}
          },
    data={'state': 'monitor_state.data',
          'process_state': 'monitor_process_resource.status',
          'process_resource': 'monitor_resource_process.data',
          'memory_resource': 'monitor_resource_memory.data',
          'io_resource': 'monitor_resource_io.data',
          'monitor_process_state': 'monitor_resource.status'},
    _embedded={'promises': promise_list},
    
  )

  instance_dict = {}
  global_state_dict['title'] = config.get('instance', 'name')
  global_state_dict['hosting-title'] = config.get('instance', 'root-name')
  if not global_state_dict['title']:
    global_state_dict['title'] = 'Instance Monitoring'

  instance_dict['computer'] = config.get('instance', 'computer')
  instance_dict['ipv4'] = config.get('instance', 'ipv4')
  instance_dict['ipv6'] = config.get('instance', 'ipv6')
  instance_dict['software-release'] = config.get('instance', 'software-release')
  instance_dict['software-type'] = config.get('instance', 'software-type')
  instance_dict['partition'] = config.get('instance', 'partition')

  global_state_dict['_embedded'].update({'instance' : instance_dict})

  if related_monitor_list:
    global_state_dict['_links']['related_monitor'] = [{'href': "%s/share/jio_public" % url}
                          for url in related_monitor_list]

  if os.path.exists(parameter_file):
    with open(parameter_file) as cfile:
      global_state_dict['parameters'] = json.loads(cfile.read())

  # Public information with the link to private folder
  public_state_dict = dict(
    status=status,
    date=report_date,
    _links={'monitor': {'href': '%s/share/jio_private/' % base_url}},
    title=global_state_dict.get('title', '')
  )
  public_state_dict['hosting-title'] = global_state_dict.get('hosting-title', '')
  public_state_dict['_links']['related_monitor'] = global_state_dict['_links'].get('related_monitor', [])

  with open(global_state_file, 'w') as fglobal:
    fglobal.write(json.dumps(global_state_dict))

  with open(public_state_file, 'w') as fpglobal:
    fpglobal.write(json.dumps(public_state_dict))

  # Save document list in a file called _document_list
  public_document_list = [os.path.splitext(file)[0]
                for file in os.listdir(status_folder) if file.endswith('.json')]
  private_document_list = [os.path.splitext(file)[0]
                  for file in os.listdir(base_folder) if file.endswith('.json')]
  data_document_list = [os.path.splitext(file)[0]
              for file in os.listdir(statistic_folder) if file.endswith('.json')]

  with open(os.path.join(status_folder, '_document_list'), 'w') as lfile:
    lfile.write('\n'.join(public_document_list))

  with open(os.path.join(base_folder, '_document_list'), 'w') as lfile:
    lfile.write('\n'.join(private_document_list))

  with open(os.path.join(statistic_folder, '_document_list'), 'w') as lfile:
    lfile.write('\n'.join(data_document_list))

  generateStatisticsData(
    os.path.join(statistic_folder, 'monitor_state.data.json'),
    global_state_dict)

  return 0

def main():
  if len(sys.argv) < 3:
    print("Usage: %s <monitor_conf_path> <instance_conf_path>" % sys.argv[0])
    sys.exit(2)
  sys.exit(run(sys.argv[1:]))



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010-2014 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import sqlite3
import os
import pwd
import time
import json
import argparse
import psutil
from time import strftime
from datetime import datetime, timedelta


def parseArguments():
  """
  Parse arguments for monitor collector instance.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--output_folder',
                      help='Path of the folder where output files should be written.')
  parser.add_argument('--partition_id',
                      help='ID of the computer partition to collect data from.')
  parser.add_argument('--collector_db',
                      help='The path of slapos collect database.')

  return parser.parse_args()


class ResourceCollect:

  def __init__(self, db_path = None):
    assert os.path.exists(db_path) and os.path.isfile(db_path)
    self.uri = db_path
    self.connection = None
    self.cursor = None

  def connect(self):
    self.connection = sqlite3.connect(self.uri)
    self.cursor = self.connection.cursor()

  def close(self):
    assert self.connection is not None
    self.cursor.close()
    self.connection.close()

  def _execute(self, sql):
    assert self.connection is not None
    return self.cursor.execute(sql)

  def select(self, table, date=None, columns="*", where=None):
    """ Query database for a full table information """
    if date is not None:
      where_clause = " WHERE date = '%s' " % date
    else:
      where_clause = ""
    
    if where is not None:
      if where_clause == "":
        where_clause += " WHERE 1 = 1 "
      where_clause += " AND %s " % where 
    select_sql = "SELECT %s FROM %s %s " % (columns, table, where_clause)
    return self._execute(select_sql)

  def has_table(self, name):
    self.connect()
    check_result_cursor = self.select(
      table="sqlite_master",
      columns='name',
      where="type='table' AND name='%s'" % name)
    table_exists_result = zip(*check_result_cursor)
    if not len(table_exists_result) or table_exists_result[0][0] is None:
      return False
    return True

  def getPartitionCPULoadAverage(self, partition_id, date_scope):
    self.connect()
    query_result_cursor = self.select("user", date_scope,
                       columns="SUM(cpu_percent)", 
                       where="partition = '%s'" % partition_id)

    cpu_percent_sum = zip(*query_result_cursor)
    if len(cpu_percent_sum) and cpu_percent_sum[0][0] is None:
      return

    query_result_cursor = self.select("user", date_scope,
                       columns="COUNT(DISTINCT time)", 
                       where="partition = '%s'" % partition_id)

    sample_amount = zip(*query_result_cursor)
    self.close()

    if len(sample_amount) and len(cpu_percent_sum):
      return round(cpu_percent_sum[0][0]/sample_amount[0][0], 2)

  def getPartitionUsedMemoryAverage(self, partition_id, date_scope):
    self.connect()
    query_result_cursor = self.select("user", date_scope,
                       columns="SUM(memory_rss)", 
                       where="partition = '%s'" % partition_id)

    memory_sum = zip(*query_result_cursor)
    if len(memory_sum) and memory_sum[0][0] is None:
      return

    query_result_cursor = self.select("user", date_scope,
                       columns="COUNT(DISTINCT time)", 
                       where="partition = '%s'" % partition_id)

    sample_amount = zip(*query_result_cursor)
    self.close()

    if len(sample_amount) and len(memory_sum):
      return round(memory_sum[0][0]/(sample_amount[0][0]*1024*1024.0), 2)

  def getPartitionDiskUsedAverage(self, partition_id, date_scope):
    if not self.has_table('folder'):
      return
    self.db.connect()
    query_result_cursor = self.select("folder", date_scope,
                       columns="SUM(disk_used)", 
                       where="partition = '%s'" % partition_id)

    disk_used_sum = zip(*query_result_cursor)
    if len(disk_used_sum) and disk_used_sum[0][0] is None:
      return
    query_result_cursor = self.select("folder", date_scope,
                       columns="COUNT(DISTINCT time)", 
                       where="partition = '%s'" % partition_id)
  
    collect_amount = zip(*query_result_cursor)
    self.db.close()
  
    if len(collect_amount) and len(disk_used_sum):
      return round(disk_used_sum[0][0]/(collect_amount[0][0]*1024.0), 2)


  def getPartitionConsumption(self, partition_id, where=""):
    """
      Query collector db to get consumed resource for last minute
    """
    self.connect()
    comsumption_list = []
    if where != "":
      where = "and %s" % where
    date_scope = datetime.now().strftime('%Y-%m-%d')
    min_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:00')
    max_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:59')

    sql_query = """select count(pid), SUM(cpu_percent) as cpu_result, SUM(cpu_time), 
MAX(cpu_num_threads), SUM(memory_percent), SUM(memory_rss), pid, SUM(io_rw_counter), 
SUM(io_cycles_counter) from user 
where date='%s' and partition='%s' and (time between '%s' and '%s') %s 
group by pid order by cpu_result desc""" % (
        date_scope, partition_id, min_time, max_time, where)
    query_result = self._execute(sql_query)
    for result in query_result:
      count = int(result[0])
      if not count > 0:
        continue
      resource_dict = {
        'pid': result[6],
        'cpu_percent': round(result[1]/count, 2),
        'cpu_time': round((result[2] or 0)/(60.0), 2),
        'cpu_num_threads': round(result[3]/count, 2),
        'memory_percent': round(result[4]/count, 2),
        'memory_rss': round((result[5] or 0)/(1024*1024.0), 2),
        'io_rw_counter': round(result[7]/count, 2),
        'io_cycles_counter': round(result[8]/count, 2)
      }
      try:
        pprocess = psutil.Process(int(result[6]))
      except psutil.NoSuchProcess:
        pass
      else:
        resource_dict['name'] = pprocess.name()
        resource_dict['command'] = pprocess.cmdline()
        resource_dict['user'] = pprocess.username()
        resource_dict['date'] = datetime.fromtimestamp(pprocess.create_time()).strftime("%Y-%m-%d %H:%M:%S")
      comsumption_list.append(resource_dict)
    self.close()
    return comsumption_list
  
  def getPartitionComsumptionStatus(self, partition_id, where=""):
    self.connect()
    if where != "":
      where = " and %s" % where
    date_scope = datetime.now().strftime('%Y-%m-%d')
    min_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:00')
    max_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:59')
    sql_query = """select count(pid), SUM(cpu_percent), SUM(cpu_time), 
SUM(cpu_num_threads), SUM(memory_percent), SUM(memory_rss), SUM(io_rw_counter), 
SUM(io_cycles_counter) from user where 
date='%s' and partition='%s' and (time between '%s' and '%s') %s""" % (
        date_scope, partition_id, min_time, max_time, where)
    query_result = self._execute(sql_query)
    result_list = zip(*query_result)

    process_dict = memory_dict = io_dict = {}
    if len(result_list):
      result = result_list
      process_dict = {'total_process': result[0][0],
        'cpu_percent': round((result[1][0] or 0), 2),
        'cpu_time': round((result[2][0] or 0)/(60.0), 2),
        'cpu_num_threads': round((result[3][0] or 0), 2),
        'date': '%s %s' % (date_scope, min_time)
      }
      memory_dict = {'memory_percent': round((result[4][0] or 0), 2),
        'memory_rss': round((result[5][0] or 0)/(1024*1024.0), 2),
        'date': '%s %s' % (date_scope, min_time)
      }
      io_dict = {'io_rw_counter': round((result[6][0] or 0), 2),
        'io_cycles_counter': round((result[7][0] or 0), 2),
        'disk_used': 0,
        'date': '%s %s' % (date_scope, min_time)
      }
      if self.has_table('folder'):
        disk_result_cursor = self.select(
          "folder", date_scope,
          columns="SUM(disk_used)", 
          where="partition='%s' and (time between '%s' and '%s') %s" % (
            partition_id, min_time, max_time, where
          )
        )
    
        disk_used_sum = zip(*disk_result_cursor)
        if len(disk_used_sum) and disk_used_sum[0][0] is not None:
          io_dict['disk_used'] = round(disk_used_sum[0][0]/1024.0, 2)
    self.close()
    return (process_dict, memory_dict, io_dict)

def appendToJsonFile(file_path, content, stepback=2):
  with open (file_path, mode="r+") as jfile:
    jfile.seek(0, 2)
    position = jfile.tell() - stepback
    jfile.seek(position)
    jfile.write('%s}' % ',"{}"]'.format(content))

def initProcessDataFile(process_file):
  with open(process_file, 'w') as fprocess:
    data_dict = {
      "date": time.time(),
      "data": ["date, total process, CPU percent, CPU time, CPU threads"]
    }
    fprocess.write(json.dumps(data_dict))

def initMemoryDataFile(mem_file):
  with open(mem_file, 'w') as fmem:
    data_dict = {
      "date": time.time(),
      "data": ["date, memory used percent, memory used"]
    }
    fmem.write(json.dumps(data_dict))

def initIODataFile(io_file):
  with open(io_file, 'w') as fio:
    data_dict = {
      "date": time.time(),
      "data": ["date, io rw counter, io cycles counter, disk used"]
    }
    fio.write(json.dumps(data_dict))

def main():
  parser = parseArguments()
  if not os.path.exists(parser.output_folder) and os.path.isdir(parser.output_folder):
    raise Exception("Invalid ouput folder: %s" % parser.output_folder)

  # Consumption global status
  process_file = os.path.join(parser.output_folder, 'monitor_resource_process.data.json')
  mem_file = os.path.join(parser.output_folder, 'monitor_resource_memory.data.json')
  io_file = os.path.join(parser.output_folder, 'monitor_resource_io.data.json')
  resource_file = os.path.join(parser.output_folder, 'monitor_process_resource.status.json')
  status_file = os.path.join(parser.output_folder, 'monitor_resource.status.json')

  if not os.path.exists(parser.collector_db):
    print "Collector database not found..."
    initProcessDataFile(process_file)
    initMemoryDataFile(mem_file)
    initIODataFile(io_file)
    with open(status_file, "w") as status_file:
      status_file.write('{"cpu_time": 0, "cpu_percent": 0, "memory_rss": 0, "memory_percent": 0, "io_rw_counter": 0, "date": "", "total_process": 0, "disk_used": 0, "io_cycles_counter": 0, "cpu_num_threads": 0}')
    with open(resource_file, "w") as resource_file:
      resource_file.write('[]')
    exit(1)

  collector = ResourceCollect(parser.collector_db)

  date_scope = datetime.now().strftime('%Y-%m-%d')
  stat_info = os.stat(parser.output_folder)
  partition_user = pwd.getpwuid(stat_info.st_uid)[0]

  process_result, memory_result, io_result = collector.getPartitionComsumptionStatus(partition_user)

  label_list = ['date', 'total_process', 'cpu_percent', 'cpu_time', 'cpu_num_threads',
                  'memory_percent', 'memory_rss', 'io_rw_counter', 'io_cycles_counter',
                  'disk_used']
  resource_status_dict = {}
  if not os.path.exists(process_file) or os.stat(process_file).st_size == 0:
    initProcessDataFile(process_file)

  if not os.path.exists(mem_file) or os.stat(mem_file).st_size == 0:
    initMemoryDataFile(mem_file)

  if not os.path.exists(io_file) or os.stat(io_file).st_size == 0:
    initIODataFile(io_file)

  if process_result and process_result['total_process'] != 0.0:
    appendToJsonFile(process_file, ", ".join(
      [str(process_result[key]) for key in label_list if process_result.has_key(key)])
    )
    resource_status_dict.update(process_result)
  if memory_result and memory_result['memory_rss'] != 0.0:
    appendToJsonFile(mem_file, ", ".join(
      [str(memory_result[key]) for key in label_list if memory_result.has_key(key)])
    )
    resource_status_dict.update(memory_result)
  if io_result and io_result['io_rw_counter'] != 0.0:
    appendToJsonFile(io_file, ", ".join(
      [str(io_result[key]) for key in label_list if io_result.has_key(key)])
    )
    resource_status_dict.update(io_result)

  with open(status_file, 'w') as fp:
    fp.write(json.dumps(resource_status_dict))

  # Consumption Resource
  resource_process_status_list = collector.getPartitionConsumption(partition_user)
  if resource_process_status_list:
    with open(resource_file, 'w') as rf:
      rf.write(json.dumps(resource_process_status_list))



# -*- coding: utf-8 -*-
import os, time
import sys
import shutil
import tempfile
import unittest
import json
from datetime import datetime
from slapos.monitor.runpromise import RunPromise, parseArguments

class MonitorPromiseTest(unittest.TestCase):

  def setUp(self):
    self.base_dir = tempfile.mkdtemp()
    self.promise_dir = os.path.join(self.base_dir, 'promise')
    self.monitor_promise_dir = os.path.join(self.base_dir, 'monitor-promise')
    self.report_dir = os.path.join(self.base_dir, 'report')
    self.public_dir = os.path.join(self.base_dir, 'public')
    self.private_dir = os.path.join(self.base_dir, 'private')
    self.run_dir = os.path.join(self.base_dir, 'run')
    os.mkdir(self.promise_dir)
    os.mkdir(self.public_dir)
    os.mkdir(self.private_dir)
    os.mkdir(self.report_dir)
    os.mkdir(self.run_dir)
    os.mkdir(self.monitor_promise_dir)


  def tearDown(self):
    if os.path.exists(self.base_dir):
      shutil.rmtree(self.base_dir)

  def writePromiseOK(self, name, monitor_folder=False):
    content = """#!/bin/sh

echo "success"
exit 0
"""
    promise_path = os.path.join(self.promise_dir, name)
    if monitor_folder:
      promise_path = os.path.join(self.monitor_promise_dir, name)
    self.writeContent(promise_path, content)
    os.chmod(promise_path, 0755)
    return promise_path

  def writePromiseNOK(self, name):
    content = """#!/bin/sh

echo "failed"
exit 2
"""
    promise_path = os.path.join(self.promise_dir, name)
    self.writeContent(promise_path, content)
    os.chmod(promise_path, 0755)
    return promise_path

  def writeContent(self, file_path, config):
    with open(file_path, 'w') as cfg:
      cfg.write(config)

  def getUniquePromiseParser(self, name, promise_path, promise_type):
    pid_path = os.path.join(self.run_dir, '%s.pid' % name)
    if promise_type == "report":
      output_path = os.path.join(self.private_dir, '%s.report.json' % name)
    else:
      output_path = os.path.join(self.public_dir, '%s.status.json' % name)
    promise_cmd = [
      '--pid_path',
      '%s' % pid_path, '--output', output_path,
      '--promise_script', promise_path,
      '--promise_name', name, '--promise_type', promise_type,
      '--monitor_url', 'https://monitor.test.com/share/jio_private/',
      '--history_folder', self.public_dir,
      '--instance_name', 'Monitor', '--hosting_name', 'Monitor ROOT']
    arg_parser = parseArguments()
    return arg_parser.parse_args(promise_cmd)

  def getPromiseParser(self):
    pid_path = os.path.join(self.run_dir, 'monitor-promise.pid')

    promise_cmd = [
      '--pid_path',
      '%s' % pid_path, '--output', self.public_dir,
      '--promise_folder', self.promise_dir,
      '--monitor_promise_folder', self.monitor_promise_dir,
      '--promise_type', 'status',
      '--monitor_url', 'https://monitor.test.com/share/jio_private/',
      '--history_folder', self.public_dir,
      '--instance_name', 'Monitor', '--hosting_name', 'Monitor ROOT']
    arg_parser = parseArguments()
    return arg_parser.parse_args(promise_cmd)

  def test_promise_OK(self):
    
    promise = self.writePromiseOK('promise_1')
    parser = self.getPromiseParser()
    promise_runner = RunPromise(parser)
    promise_runner.runpromise()

    result_file = os.path.join(self.public_dir, 'promise_1.status.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read())
    change_time = result1.pop('change-time', 0)
    change_date = datetime.fromtimestamp(change_time)
    start_date = result1.pop('start-date')
    
    expected_result = {'status': 'OK', 'hosting_subscription': 'Monitor ROOT',
      'title': u'promise_1', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'success\n', 'type': 'status'}
    self.assertEquals(expected_result, result1)

    # second run
    time.sleep(1)
    promise_runner.runpromise()
    result2 = json.loads(open(result_file).read())
    change_time2 = result2.pop('change-time', 0)
    start_date2 = result2.pop('start-date')
    change_date2 = datetime.fromtimestamp(change_time2)
    self.assertEquals(expected_result, result2)
    self.assertEquals(change_date.strftime('%Y-%m-%d %H:%M:%S'),
      change_date2.strftime('%Y-%m-%d %H:%M:%S'))

    history_file = os.path.join(self.public_dir, 'promise_1.history.json')
    self.assertTrue(os.path.exists(history_file))
    history = json.load(open(history_file))
    self.assertTrue(history['date'] > change_time)
    self.assertTrue(len(history['data']) == 2)

    result1['change-time'] = change_time
    result1['start-date'] = start_date
    result1.pop('_links')
    result2['start-date'] = start_date2
    result2['change-time'] = change_time2
    # not in history
    result2.pop('_links')
    result2.pop('hosting_subscription')
    result2.pop('title')
    result2.pop('instance')
    result2.pop('type')
    self.assertEquals(history['data'][0], result1)
    self.assertEquals(history['data'][1], result2)

  def test_promise_two_folder(self):
    
    promise = self.writePromiseOK('promise_1')
    promise2 = self.writePromiseOK('promise_2', monitor_folder=True)
    parser = self.getPromiseParser()
    promise_runner = RunPromise(parser)
    promise_runner.runpromise()

    result_file = os.path.join(self.public_dir, 'promise_1.status.json')
    result2_file = os.path.join(self.public_dir, 'promise_2.status.json')
    self.assertTrue(os.path.exists(result_file))
    self.assertTrue(os.path.exists(result2_file))
    result1 = json.loads(open(result_file).read())
    result1.pop('change-time')
    result1.pop('start-date')
    
    expected_result = {'status': 'OK', 'hosting_subscription': 'Monitor ROOT',
      'title': u'promise_1', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'success\n', 'type': 'status'}
    self.assertEquals(expected_result, result1)

    result2 = json.loads(open(result2_file).read())
    result2.pop('change-time')
    result2.pop('start-date')
    
    expected_result = {'status': 'OK', 'hosting_subscription': 'Monitor ROOT',
      'title': u'promise_2', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'success\n', 'type': 'status'}
    self.assertEquals(expected_result, result2)

  def test_promise_One_By_One(self):
    
    promise = self.writePromiseOK('promise_1')
    parser = self.getUniquePromiseParser('promise_1', promise, 'status')
    promise_runner = RunPromise(parser)
    promise_runner.runpromise()

    result_file = os.path.join(self.public_dir, 'promise_1.status.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read())
    change_time = result1.pop('change-time', 0)
    change_date = datetime.fromtimestamp(change_time)
    start_date = result1.pop('start-date')
    
    expected_result = {'status': 'OK', 'hosting_subscription': 'Monitor ROOT',
      'title': u'promise_1', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'success\n', 'type': 'status'}
    self.assertEquals(expected_result, result1)

    # second run
    promise_runner.runpromise()
    result2 = json.loads(open(result_file).read())
    change_time2 = result2.pop('change-time', 0)
    result2.pop('start-date', '2016-08-05 00:00:00')
    change_date2 = datetime.fromtimestamp(change_time2)
    self.assertEquals(expected_result, result2)
    self.assertEquals(change_date.strftime('%Y-%m-%d %H:%M:%S'),
      change_date2.strftime('%Y-%m-%d %H:%M:%S'))

    history_file = os.path.join(self.public_dir, 'promise_1.history.json')
    self.assertTrue(os.path.exists(history_file))
    history = json.load(open(history_file))
    self.assertTrue(history['date'] > change_time)
    self.assertTrue(len(history['data']) == 1)

    result1['change-time'] = change_time
    result1['start-date'] = start_date
    result1.pop('_links')
    self.assertEquals(history['data'][0], result1)
    
  def test_promise_NOK(self):
    
    promise = self.writePromiseNOK('promise_1')
    parser = self.getPromiseParser()
    promise_runner = RunPromise(parser)
    promise_runner.runpromise()

    result_file = os.path.join(self.public_dir, 'promise_1.status.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read())
    change_time = result1.pop('change-time', 0)
    change_date = datetime.fromtimestamp(change_time)
    start_date = result1.pop('start-date')
    
    expected_result = {'status': 'ERROR', 'hosting_subscription': 'Monitor ROOT',
      'title': u'promise_1', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'failed\n', 'type': 'status'}
    self.assertEquals(expected_result, result1)

    # second run
    promise_runner.runpromise()
    result2 = json.loads(open(result_file).read())
    change_time2 = result2.pop('change-time', 0)
    result2.pop('start-date', '2016-08-05 00:00:00')
    change_date2 = datetime.fromtimestamp(change_time2)
    self.assertEquals(expected_result, result2)
    self.assertEquals(change_date.strftime('%Y-%m-%d %H:%M:%S'),
      change_date2.strftime('%Y-%m-%d %H:%M:%S'))

  def test_promise_mixed(self):
    
    promise = self.writePromiseOK('promise_1')
    parser = self.getPromiseParser()
    promise_runner = RunPromise(parser)
    promise_runner.runpromise()

    result_file = os.path.join(self.public_dir, 'promise_1.status.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read())
    change_time = result1.pop('change-time')
    change_date = datetime.fromtimestamp(change_time)
    start_date = result1.pop('start-date')
    
    expected_result = {'status': 'OK', 'hosting_subscription': 'Monitor ROOT',
      'title': u'promise_1', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'success\n', 'type': 'status'}
    self.assertEquals(expected_result, result1)

    # second run with failure
    time.sleep(2)
    promise = self.writePromiseNOK('promise_1')
    parser = self.getPromiseParser()
    expected_result['message'] = 'failed\n'
    expected_result['status'] = 'ERROR'
    promise_runner.runpromise()

    result2 = json.loads(open(result_file).read())
    change_time2 = result2.pop('change-time')
    result2.pop('start-date')
    change_date2 = datetime.fromtimestamp(change_time2)
    self.assertEquals(expected_result, result2)
    self.assertNotEquals(change_date.strftime('%Y-%m-%d %H:%M:%S'),
      change_date2.strftime('%Y-%m-%d %H:%M:%S'))

  def test_report_OK(self):
    
    promise = self.writePromiseOK('sample_report')
    parser = self.getUniquePromiseParser('sample_report', promise, 'report')
    promise_runner = RunPromise(parser)
    promise_runner.runpromise()

    result_file = os.path.join(self.private_dir, 'sample_report.report.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read())
    change_time = result1.pop('change-time', 0)
    change_date = datetime.fromtimestamp(change_time)
    start_date = result1.pop('start-date')
    
    expected_result = {'status': 'OK', 'hosting_subscription': 'Monitor ROOT',
      'title': 'sample_report', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'success\n', 'type': 'report'}
    self.assertEquals(expected_result, result1)

    # second run
    promise_runner.runpromise()
    result2 = json.loads(open(result_file).read())
    change_time2 = result2.pop('change-time', 0)
    result2.pop('start-date', '2016-08-05 00:00:00')
    change_date2 = datetime.fromtimestamp(change_time2)
    self.assertEquals(expected_result, result2)
    self.assertEquals(change_date.strftime('%Y-%m-%d %H:%M:%S'),
      change_date2.strftime('%Y-%m-%d %H:%M:%S'))

    history_file = os.path.join(self.public_dir, 'sample_report.history.json')
    self.assertTrue(os.path.exists(history_file))
    history = json.load(open(history_file))


    result1['change-time'] = change_time
    result1['start-date'] = start_date
    #result1.pop('_links')
    self.assertEquals(history, result1)

  def test_report_mixed(self):
    
    promise = self.writePromiseOK('sample_report')
    parser = self.getUniquePromiseParser('sample_report', promise, 'report')
    promise_runner = RunPromise(parser)
    promise_runner.runpromise()

    result_file = os.path.join(self.private_dir, 'sample_report.report.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read())
    change_time = result1.pop('change-time', 0)
    change_date = datetime.fromtimestamp(change_time)
    start_date = result1.pop('start-date')
    
    expected_result = {'status': 'OK', 'hosting_subscription': 'Monitor ROOT',
      'title': 'sample_report', 'instance': 'Monitor',
      '_links': 
        {'monitor': {'href': 'https://monitor.test.com/share/jio_private/'}},
        'message': 'success\n', 'type': 'report'}
    self.assertEquals(expected_result, result1)

    # second run with failure
    time.sleep(2)
    promise = self.writePromiseNOK('sample_report')
    parser = self.getUniquePromiseParser('sample_report', promise, 'report')
    expected_result['message'] = 'failed\n'
    expected_result['status'] = 'ERROR'
    promise_runner.runpromise()

    result2 = json.loads(open(result_file).read())
    change_time2 = result2.pop('change-time')
    result2.pop('start-date')
    change_date2 = datetime.fromtimestamp(change_time2)
    self.assertEquals(expected_result, result2)
    self.assertNotEquals(change_date.strftime('%Y-%m-%d %H:%M:%S'),
      change_date2.strftime('%Y-%m-%d %H:%M:%S'))





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import stat
import json
import ConfigParser
import traceback
import argparse
import urllib2
import ssl
import glob
from datetime import datetime

OPML_START = """<?xml version="1.0" encoding="UTF-8"?>
<!-- OPML generated by SlapOS -->
<opml version="1.1">
	<head>
		<title>%(root_title)s</title>
		<dateCreated>%(creation_date)s</dateCreated>
		<dateModified>%(modification_date)s</dateModified>
	</head>
	<body>
	  <outline text="%(outline_title)s">"""
OPML_END = """	  </outline>
  </body>
</opml>"""

OPML_OUTLINE_FEED = '<outline text="%(title)s" title="%(title)s" type="rss" version="RSS" htmlUrl="%(html_url)s" xmlUrl="%(xml_url)s" url="%(global_url)s" />'


def parseArguments():
  """
  Parse arguments for monitor instance.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--config_file',
                      default='monitor.cfg',
                      help='Monitor Configuration file')

  return parser.parse_args()


def mkdirAll(path):
  try:
    os.makedirs(path)
  except OSError, e:
    if e.errno == os.errno.EEXIST and os.path.isdir(path):
      pass
    else: raise

def softConfigGet(config, *args, **kwargs):
  try:
    return config.get(*args, **kwargs)
  except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    return None

def createSymlink(source, destination):
  try:
    os.symlink(source, destination)
  except OSError, e:
    if e.errno != os.errno.EEXIST:
      raise

class Monitoring(object):

  def __init__(self, configuration_file):
    config = self.loadConfig([configuration_file])

    # Set Monitor variables
    self.title = config.get("monitor", "title")
    self.root_title = config.get("monitor", "root-title")
    self.service_pid_folder = config.get("monitor", "service-pid-folder")
    self.crond_folder = config.get("monitor", "crond-folder")
    self.logrotate_d = config.get("monitor", "logrotate-folder")
    self.promise_runner = config.get("monitor", "promise-runner")
    self.promise_folder = config.get("monitor", "promise-folder")
    self.public_folder = config.get("monitor", "public-folder")
    self.private_folder = config.get("monitor", "private-folder")
    self.collector_db  = config.get("monitor", "collector-db")
    self.collect_script = config.get("monitor", "collect-script")
    self.statistic_script = config.get("monitor", "statistic-script")
    self.webdav_folder = config.get("monitor", "webdav-folder")
    self.report_script_folder = config.get("monitor", "report-folder")
    self.webdav_url = '%s/share' % config.get("monitor", "base-url")
    self.public_url = '%s/public' % config.get("monitor", "base-url")
    self.python = config.get("monitor", "python") or "python"
    self.public_path_list = config.get("monitor", "public-path-list").split()
    self.private_path_list = config.get("monitor", "private-path-list").split()
    self.monitor_url_list = config.get("monitor", "monitor-url-list").split()
    self.parameter_list = [param.strip() for param in config.get("monitor", "parameter-list").split('\n') if param]
    # Use this file to write knowledge0_cfg required by webrunner
    self.parameter_cfg_file = config.get("monitor", "parameter-file-path").strip()
    self.pid_file = config.get("monitor", "pid-file")
    self.monitor_promise_folder = softConfigGet(config, "monitor",
                                                "monitor-promise-folder")
    self.promise_timeout_file = softConfigGet(config, "monitor",
                                                "promises-timeout-file")
    self.nice_command = softConfigGet(config, "monitor",
                                                "nice-cmd")

    self.config_folder = os.path.join(self.private_folder, 'config')
    self.report_folder = self.private_folder

    self.promise_output_file = config.get("monitor", "promise-output-file")
    self.bootstrap_is_ok = True

    if self.nice_command:
      # run monitor promises script with low priority
      self.promise_runner = '%s %s' % (self.nice_command, self.promise_runner)

  def loadConfig(self, pathes, config=None):
    if config is None:
      config = ConfigParser.ConfigParser()
    try:
      config.read(pathes)
    except ConfigParser.MissingSectionHeaderError:
      traceback.print_exc()
    return config

  def readInstanceConfiguration(self):
    type_list = ['raw', 'file', 'htpasswd', 'httpdcors']
    configuration_list = []

    if not self.parameter_list:
      return []
  
    for config in self.parameter_list:
      config_list = config.strip().split(' ')
      # type: config_list[0]
      if len(config_list) >= 3 and config_list[0] in type_list:
        if config_list[0] == 'raw':
          configuration_list.append(dict(
            key='',
            title=config_list[1],
            value=' '.join(config_list[2:])
          ))
        elif (config_list[0] == 'file' or config_list[0] == 'htpasswd')  and \
            os.path.exists(config_list[2]) and os.path.isfile(config_list[2]):
          try:
            with open(config_list[2]) as cfile:
              parameter = dict(
                key=config_list[1],
                title=config_list[1],
                value=cfile.read(),
                description={
                  "type": config_list[0],
                  "file": config_list[2]
                }
              )
              if config_list[0] == 'htpasswd':
                if len(config_list) != 5 or not os.path.exists(config_list[4]):
                  print 'htpasswd file is not specified: %s' % str(config_list)
                  continue
                parameter['description']['user'] = config_list[3]
                parameter['description']['htpasswd'] = config_list[4]
              configuration_list.append(parameter)
          except OSError, e:
            print 'Cannot read file %s, Error is: %s' % (config_list[2], str(e))
            pass
        elif config_list[0] == 'httpdcors' and os.path.exists(config_list[2]) and \
            os.path.exists(config_list[3]):
          old_cors_file = os.path.join(
            os.path.dirname(config_list[2]),
            'prev_%s' % os.path.basename(config_list[2])
          )
          try:
            cors_content = ""
            if os.path.exists(old_cors_file):
              with open(old_cors_file) as cfile:
                cors_content = cfile.read()
            else:
              # Create empty file
              with open(old_cors_file, 'w') as cfile:
                cfile.write("")
            parameter = dict(
              key=config_list[1],
              title=config_list[1],
              value=cors_content,
              description={
                "type": config_list[0],
                "cors_file": config_list[2],
                "gracefull_bin": config_list[3]
              }
            )
            configuration_list.append(parameter)
          except OSError, e:
            print 'Cannot read file at %s, Error is: %s' % (old_cors_file, str(e))
            pass
    return configuration_list

  def createSymlinksFromConfig(self, destination_folder, source_path_list, name=""):
    if destination_folder:
      if source_path_list:
        for path in source_path_list:
          path = path.rstrip('/')
          dirname = os.path.join(destination_folder, name)
          try:
            mkdirAll(dirname)  # could also raise OSError
            os.symlink(path, os.path.join(dirname, os.path.basename(path)))
          except OSError, e:
            if e.errno != os.errno.EEXIST:
              raise

  def getMonitorTitleFromUrl(self, monitor_url):
    # This file should be generated
    if not monitor_url.startswith('https://') and not monitor_url.startswith('http://'):
      return 'Unknown Instance'
    if not monitor_url.endswith('/'):
      monitor_url = monitor_url + '/'

    url  = monitor_url + '/.jio_documents/monitor.global.json' # XXX Hard Coded path
    try:
      # XXX - working here with public url
      if hasattr(ssl, '_create_unverified_context'):
        context = ssl._create_unverified_context()
        response = urllib2.urlopen(url, context=context)
      else:
        response = urllib2.urlopen(url)
    except urllib2.HTTPError:
      self.bootstrap_is_ok = False
      print "Error: Failed to get Monitor configuration at %s " % monitor_url
      return 'Unknown Instance'
    else:
      try:
        monitor_dict = json.loads(response.read())
        return monitor_dict.get('title', 'Unknown Instance')
      except ValueError, e:
        print "Bad Json file at %s" % url
        self.bootstrap_is_ok = False
    return 'Unknown Instance'

  def getReportInfoFromFilename(self, filename):
    splited_filename = filename.split('_every_')
    possible_time_list = ['hour', 'minute']
    if len(splited_filename) == 1:
      return (filename, "* * * * *")

    run_time = splited_filename[1].split('_')
    report_name = splited_filename[0]
    if len(run_time) != 2 or not run_time[1] in possible_time_list:
      return (report_name, "* * * * *")

    try:
      value = int(run_time[0])
    except ValueError:
      print "Warning: Bad report filename: %s" % filename
      return (report_name, "* * * * *")

    if run_time[1] == 'hour':
      return (report_name, "11 */%s * * *" % value)
    if run_time[1] == 'minute':
      return (report_name, "*/%s * * * *" % value)

  def configureFolders(self):
    # configure public and private folder
    self.createSymlinksFromConfig(self.webdav_folder, [self.public_folder])
    self.createSymlinksFromConfig(self.webdav_folder, [self.private_folder])

    #configure jio_documents folder
    jio_public = os.path.join(self.webdav_folder, 'jio_public')
    jio_private = os.path.join(self.webdav_folder, 'jio_private')
    mkdirAll(jio_public)
    mkdirAll(jio_private)

    createSymlink(self.public_folder,
                  os.path.join(jio_public, '.jio_documents'))
    createSymlink(self.private_folder,
                  os.path.join(jio_private, '.jio_documents'))

    self.data_folder = os.path.join(self.private_folder, 'data', '.jio_documents')
    self.document_folder = os.path.join(self.private_folder, 'documents')
    config_folder = os.path.join(self.config_folder, '.jio_documents')

    mkdirAll(self.data_folder)
    mkdirAll(config_folder)

    createSymlink(os.path.join(self.private_folder, 'data'),
                  os.path.join(jio_private, 'data'))
    createSymlink(self.config_folder, os.path.join(jio_private, 'config'))
    createSymlink(self.data_folder, self.document_folder)

    # Cleanup private folder
    for file in glob.glob("%s/*.history.json" % self.private_folder):
      try:
        os.unlink(file)
      except OSError:
        print "failed to remove file %s. Ignoring..." % file

  def makeConfigurationFiles(self):
    config_folder = os.path.join(self.config_folder, '.jio_documents')
    parameter_config_file = os.path.join(config_folder, 'config.parameters.json')
    parameter_file = os.path.join(config_folder, 'config.json')
    #mkdirAll(config_folder)

    parameter_list = self.readInstanceConfiguration()
    description_dict = {}

    if parameter_list:
      for i in range(0, len(parameter_list)):
        key = parameter_list[i]['key']
        if key:
          description_dict[key] = parameter_list[i].pop('description')

    with open(parameter_config_file, 'w') as config_file:
      config_file.write(json.dumps(description_dict))

    with open(parameter_file, 'w') as config_file:
      config_file.write(json.dumps(parameter_list))

    try:
      with open(self.parameter_cfg_file, 'w') as pfile:
        pfile.write('[public]\n')
        for parameter in parameter_list:
          if parameter['key']:
            pfile.write('%s = %s\n' % (parameter['key'], parameter['value']))
    except OSError, e:
      print "Error failed to create file %s" % self.parameter_cfg_file
      pass
      

  def generateOpmlFile(self, feed_url_list, output_file):

    if os.path.exists(output_file):
      creation_date = datetime.fromtimestamp(os.path.getctime(output_file)).utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")
      modification_date = datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")
    else:
      creation_date = modification_date = datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")

    opml_content = OPML_START % {'creation_date': creation_date,
                                  'modification_date': modification_date,
                                  'outline_title': 'Monitoring RSS Feed list',
                                  'root_title': self.root_title}

    opml_content += OPML_OUTLINE_FEED % {'title': self.title,
        'html_url': self.public_url + '/feed',
        'xml_url': self.public_url + '/feed',
        'global_url': "%s/jio_private/" % self.webdav_url}
    for feed_url in feed_url_list:
      opml_content += OPML_OUTLINE_FEED % {'title': self.getMonitorTitleFromUrl(feed_url + "/share/jio_public/"),
        'html_url': feed_url + '/public/feed',
        'xml_url': feed_url + '/public/feed',
        'global_url': "%s/share/jio_private/" % feed_url}

    opml_content += OPML_END

    with open(output_file, 'w') as wfile:
      wfile.write(opml_content)

  def generateLogrotateEntry(self, name, file_list, option_list):
    """
      Will add a new entry in logrotate.d folder. This can help to rotate data file daily
    """
    content = "%(logfiles)s {\n%(options)s\n}\n" % {
                'logfiles': ' '.join(file_list),
                'options': '\n'.join(option_list)
              }
    file_path = os.path.join(self.logrotate_d, name)
    with open(file_path, 'w') as flog:
      flog.write(content)

  def generateReportCronEntries(self):
    cron_line_list = []

    report_name_list = [name.replace('.report.json', '')
      for name in os.listdir(self.report_folder) if name.endswith('.report.json')]

    for filename in os.listdir(self.report_script_folder):
      report_script = os.path.join(self.report_script_folder, filename)
      if os.path.isfile(report_script) and os.access(report_script, os.X_OK):
        report_name, frequency = self.getReportInfoFromFilename(filename)
        # report_name = os.path.splitext(filename)[0]
        report_json_path = "%s.report.json" % report_name

        report_cmd_line = [
          frequency,
          self.promise_runner,
          '--pid_path "%s"' % os.path.join(self.service_pid_folder,
            "%s.pid" % filename),
          '--output "%s"' % os.path.join(self.report_folder,report_json_path),
          '--promise_script "%s"' % report_script,
          '--promise_name "%s"' % report_name,
          '--monitor_url "%s/jio_private/"' % self.webdav_url, # XXX hardcoded,
          '--history_folder "%s"' % self.data_folder,
          '--instance_name "%s"' % self.title,
          '--hosting_name "%s"' % self.root_title,
          '--promise_type "report"']

        cron_line_list.append(' '.join(report_cmd_line))

      if report_name in report_name_list:
        report_name_list.pop(report_name_list.index(report_name))

    # cleanup removed report json result
    if report_name_list != []:
      for report_name in report_name_list:
        result_path = os.path.join(self.report_folder, '%s.report.json' % report_name)
        if os.path.exists(result_path):
          try:
            os.unlink(result_path)
          except OSError, e:
            print "Error: Failed to delete %s" % result_path, str(e)
            pass

    with open(self.crond_folder + "/monitor-reports", "w") as freport:
      freport.write("\n".join(cron_line_list))

  def generateServiceCronEntries(self):
    # XXX only if at least one configuration file is modified, then write in the cron

    service_name_list = [name.replace('.status.json', '')
      for name in os.listdir(self.public_folder) if name.endswith('.status.json')]

    promise_cmd_line = [
      "* * * * *",
      "sleep $((1 + RANDOM % 20)) &&", # Sleep between 1 to 20 seconds
      self.promise_runner,
      '--pid_path "%s"' % os.path.join(self.service_pid_folder,
        "monitor-promises.pid"),
      '--output "%s"' % self.public_folder,
      '--promise_folder "%s"' % self.promise_folder,
      '--timeout_file "%s"' % self.promise_timeout_file,
      '--monitor_promise_folder "%s"' % self.monitor_promise_folder,
      '--monitor_url "%s/jio_private/"' % self.webdav_url, # XXX hardcoded,
      '--history_folder "%s"' % self.public_folder,
      '--instance_name "%s"' % self.title,
      '--hosting_name "%s"' % self.root_title]

    registered_promise_list = os.listdir(self.promise_folder)
    registered_promise_list.extend(os.listdir(self.monitor_promise_folder))
    delete_promise_list = []
    for service_name in service_name_list:
      if not service_name in registered_promise_list:
        delete_promise_list.append(service_name)

    if delete_promise_list != []:
      # XXX Some service was removed, delete his status file so monitor will not consider the status anymore
      for service_name in delete_promise_list:
        status_path = os.path.join(self.public_folder, '%s.status.json' % service_name)
        if os.path.exists(status_path):
          try:
            os.unlink(status_path)
          except OSError, e:
            print "Error: Failed to delete %s" % status_path, str(e)
            pass

    with open(self.crond_folder + "/monitor-promises", "w") as fp:
      fp.write(' '.join(promise_cmd_line))

  def addCronEntry(self, name, frequency, command):
    entry_line = '%s %s' % (frequency, command)
    cron_entry_file = os.path.join(self.crond_folder, name)
    with open(cron_entry_file, "w") as cronf:
      cronf.write(entry_line)

  def bootstrapMonitor(self):

    if os.path.exists(self.promise_output_file):
      os.unlink(self.promise_output_file)

    # save pid of current process into file
    with open(self.pid_file, 'w') as pid_file:
      pid_file.write(str(os.getpid()))

    # create symlinks from monitor.conf
    self.createSymlinksFromConfig(self.public_folder, self.public_path_list)
    self.createSymlinksFromConfig(self.private_folder, self.private_path_list)

    self.configureFolders()

    # Generate OPML file
    self.generateOpmlFile(self.monitor_url_list,
      os.path.join(self.public_folder, 'feeds'))

    # put promises to a cron file
    self.generateServiceCronEntries()

    # put report script to cron
    self.generateReportCronEntries()

    # Generate parameters files and scripts
    self.makeConfigurationFiles()

    # Rotate monitor data files
    option_list = [
      'daily', 'nocreate', 'olddir %s' % self.data_folder, 'rotate 5',
      'nocompress', 'missingok', 'extension .json', 'dateext',
      'dateformat -%Y-%m-%d', 'notifempty'
    ]
    file_list = [
      "%s/*.data.json" % self.private_folder,
      "%s/*.data.json" % self.data_folder]
    self.generateLogrotateEntry('monitor.data', file_list, option_list)

    # Rotate public history status file, delete data of previous days
    option_list = [
      'daily', 'nocreate', 'rotate 0',
      'nocompress', 'notifempty', 'prerotate',
      '   %s --history_folder %s' % (self.statistic_script, self.public_folder), 
      'endscript'
    ]
    file_list = ["%s/*.history.json" % self.public_folder]
    self.generateLogrotateEntry('monitor.service.status', file_list, option_list)

    # Add cron entry for SlapOS Collect
    command = "sleep $((1 + RANDOM % 60)) && " # Random sleep between 1 to 60 seconds
    if self.nice_command:
      # run monitor collect with low priority
      command += '%s ' % self.nice_command
    command += "%s --output_folder %s --collector_db %s" % (
      self.collect_script, self.data_folder, self.collector_db)
    self.addCronEntry('monitor_collect', '* * * * *', command)

    # Write an empty file when monitor bootstrap went until the end
    if self.bootstrap_is_ok:
      with open(self.promise_output_file, 'w') as promise_file:
        promise_file.write("")

    return 0


def main():
  parser = parseArguments()

  monitor = Monitoring(parser.config_file)
  
  sys.exit(monitor.bootstrapMonitor())



##############################################################################
#
# Copyright (c) 2013 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import argparse
import json
import os
import pprint
import socket
import time

def parseArgument():
  """
  Very basic argument parser. Might blow up for anything else than
  "./executable mysocket.sock stop/resume".
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--suspend', action='store_const', dest='action', const='suspend')
  parser.add_argument('--resume', action='store_const', dest='action', const='resume')
  parser.add_argument('--create-snapshot', action='store_const', dest='action', const='createSnapshot')
  parser.add_argument('--create-internal-snapshot', action='store_const', dest='action', const='createInternalSnapshot')
  parser.add_argument('--delete-internal-snapshot', action='store_const', dest='action', const='deleteInternalSnapshot')
  parser.add_argument('--drive-backup', action='store_const', dest='action', const='driveBackup') 
  parser.add_argument('--query-commands', action='store_const', dest='action', const='queryCommands')

  parser.add_argument('--socket', dest='unix_socket_location', required=True)
  parser.add_argument('remainding_argument_list', nargs=argparse.REMAINDER)
  args = parser.parse_args()
  return args.unix_socket_location, args.action, args.remainding_argument_list


class QemuQMPWrapper(object):
  """
  Small wrapper around Qemu's QMP to control a qemu VM.
  See http://git.qemu.org/?p=qemu.git;a=blob;f=qmp-commands.hx for
  QMP API definition.
  """
  def __init__(self, unix_socket_location):
    self.socket = self.connectToQemu(unix_socket_location)
    self.capabilities()

  @staticmethod
  def connectToQemu(unix_socket_location):
    """
    Create a socket, connect to qemu, be sure it answers, return socket.
    """
    if not os.path.exists(unix_socket_location):
      raise Exception('unix socket %s does not exist.' % unix_socket_location)

    print 'Connecting to qemu...'
    so = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    connected = False
    while not connected:
      try:
        so.connect(unix_socket_location)
      except socket.error:
        time.sleep(1)
        print 'Could not connect, retrying...'
      else:
        connected = True
    so.recv(1024)

    return so


  def _send(self, message):
    self.socket.send(json.dumps(message))
    data = self.socket.recv(65535)
    try:
      return json.loads(data)
    except ValueError:
      # if error the raise
      if "error" in data:
        raise Exception('ERROR: %s' % data)
      else:
        print 'Wrong data: %s' % data

  def _getVMStatus(self):
    response = self._send({'execute': 'query-status'})
    if response:
      return self._send({'execute': 'query-status'})['return']['status']
    else:
      raise IOError('Empty answer')

  def _waitForVMStatus(self, wanted_status):
    while True:
      try:
        actual_status = self._getVMStatus()
        if actual_status == wanted_status:
          return
        else:
          print 'VM in %s status, wanting it to be %s, retrying...' % (
              actual_status, wanted_status)
          time.sleep(1)
      except IOError:
          print 'VM not ready, retrying...'


  def capabilities(self):
    print 'Asking for capabilities...'
    self._send({'execute': 'qmp_capabilities'})

  def suspend(self):
    print 'Suspending VM...'
    self._send({'execute': 'stop'})
    self._waitForVMStatus('paused')

  def resume(self):
    print 'Resuming VM...'
    self._send({'execute': 'cont'})
    self._waitForVMStatus('running')

  def _queryBlockJobs(self, device):
    return self._send({'execute': 'query-block-jobs'})

  def _getRunningJobList(self, device):
    result = self._queryBlockJobs(device)

    while result is None:
      # If result is None retry to command until if return something.
      time.sleep(3)
      result = self._queryBlockJobs(device)
    if result.get('return'):
      return result['return']
    else:
      return

  def driveBackup(self, backup_target, source_device='virtio0', sync_type='full'):
    print 'Asking Qemu to perform backup to %s' % backup_target
    # XXX: check for error
    self._send({
        'execute': 'drive-backup',
        'arguments': {
            'device': source_device,
            'sync': sync_type,
            'target': backup_target,
         }
    })
    while self._getRunningJobList(backup_target):
      print 'Job is not finished yet.'
      time.sleep(20)

  def createSnapshot(self, snapshot_file, device='virtio0'):
    print self._send({
        'execute': 'blockdev-snapshot-sync',
        'arguments': {
            'device': device,
            'snapshot-file': snapshot_file,
         }
    })

  def createInternalSnapshot(self, name=None, device='virtio0'):
    if name is None:
      name = int(time.time())
    self._send({
        'execute': 'blockdev-snapshot-internal-sync',
        'arguments': {
            'device': device,
            'name': name,
         }
    })

  def deleteInternalSnapshot(self, name, device='virtio0'):
    self._send({
        'execute': 'blockdev-snapshot-delete-internal-sync',
        'arguments': {
            'device': device,
            'name': name,
         }
    })

  def queryCommands(self):
    pprint.pprint(self._send({'execute': 'query-commands'})['return'])

def main():
  unix_socket_location, action, remainding_argument_list = parseArgument()
  qemu_wrapper = QemuQMPWrapper(unix_socket_location)

  if remainding_argument_list:
    getattr(qemu_wrapper, action)(*remainding_argument_list)
  else:
    getattr(qemu_wrapper, action)()

if __name__ == '__main__':
  main()




import argparse
import datetime
import os
import re
import time
import PyRSS2Gen as RSS2

from collections import OrderedDict

def parseArguments():
  """
  Parse arguments for rdiff-backup statistics Rss Generator.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--rdiff_backup_data_folder',
                      help='Path where to find rdiff-backup statistical files')
  parser.add_argument('--output',
                      help='Path where to save the feed')
  parser.add_argument('--feed_url',
                      help='Url of this feed file.')

  return parser.parse_args()

def makeDictFromStatFile(text_content):
  m = re.search("([a-zA-Z]*) ([0-9 :.]*) \(([a-zA-Z0-9 :.]*)\)", text_content)
  if m:
    return {'metric': m.group(1),
            'value': m.group(2),
            'human_readable_value': m.group(3)}
  m = re.search("([a-zA-Z]*) ([0-9]*)", text_content)
  if m:
    return {'metric': m.group(1),
            'value': m.group(2),
            'human_readable_value': m.group(2)}

def getRSSItemFromDict(item, option):
  description = "Metric;Value;Human Readable Value\n"
  for entry in item:
    description += "%s;%s;%s\n" % (entry['metric'], entry['value'], entry['human_readable_value'])
    if entry['metric'] == "EndTime":
      pubDate = datetime.datetime.fromtimestamp(float(entry['value']))
  return RSS2.RSSItem(
    title="Rdiff-Backup Transfer Statistics",
    link=option.feed_url,
    pubDate=pubDate,
    description=description)

def genRSS(option):
  """
  Read statistics file from rdiff-backup and generate a RSS feed entry from it
  """
  stat_file_list = sorted([file for file in os.listdir(option.rdiff_backup_data_folder)
                           if file.startswith('session_statistics')])

  item_dict = OrderedDict()
  for stat_file in stat_file_list:
    with open(os.path.join(option.rdiff_backup_data_folder, stat_file), 'r') as file:
      item_dict[stat_file] = [makeDictFromStatFile(line.strip()) for line in file.readlines()]
  title="Rdiff-Backup Statistics"
  rss_feed = RSS2.RSS2(
    title=title,
    link=option.feed_url,
    description=title,
    items = [getRSSItemFromDict(item_dict[item], option) for item in item_dict])

  return rss_feed.to_xml()

def main():
  option = parseArguments()
  feed = genRSS(option)

  with open(option.output, 'w') as rss_file:
    rss_file.write(feed)

  exit(0)



# -*- coding: utf-8 -*-
# vim: set et sts=2:
# pylint: disable-msg=W0311,C0301,C0103,C0111,R0904,R0903

import ConfigParser
import datetime
import flask
import logging
import logging.handlers
import os
import urlparse
from slapos.htpasswd import HtpasswdFile
from slapos.runner.process import setHandler
import sys
from slapos.runner.utils import runInstanceWithLock
from slapos.runner.views import *
from slapos.runner.gittools import cloneRepo, switchBranch
from git import GitCommandError
import time
import traceback

TRUE_VALUES = (1, '1', True, 'true', 'True')

class Config:
  def __init__(self):
    self.configuration_file_path = None
    self.console = None
    self.log_file = None
    self.logger = None
    self.verbose = None

  def setConfig(self):
    """
    Set options given by parameters.
    """
    self.configuration_file_path = os.path.abspath(os.getenv('RUNNER_CONFIG'))

    # Load configuration file
    configuration_parser = ConfigParser.SafeConfigParser()
    configuration_parser.read(self.configuration_file_path)

    for section in ("slaprunner", "slapos", "slapproxy", "slapformat",
                    "sshkeys_authority", "gitclient"):
      configuration_dict = dict(configuration_parser.items(section))
      for key in configuration_dict:
        if not getattr(self, key, None):
          setattr(self, key, configuration_dict[key])

    # set up logging
    self.logger = logging.getLogger("slaprunner")
    self.logger.setLevel(logging.INFO)
    if self.console:
      self.logger.addHandler(logging.StreamHandler())

    self.log_file = self.log_dir + '/slaprunner.log'
    if not os.path.isdir(os.path.dirname(self.log_file)):
    # fallback to console only if directory for logs does not exists and
    # continue to run
      raise ValueError('Please create directory %r to store %r log file' % (
      os.path.dirname(self.log_file), self.log_file))
    else:
      file_handler = logging.FileHandler(self.log_file)
      file_handler.setFormatter(logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s"))
      self.logger.addHandler(file_handler)
      self.logger.info('Configured logging to file %r' % self.log_file)

    self.logger.info("Started.")
    self.logger.info(os.environ['PATH'])
    if self.verbose:
      self.logger.setLevel(logging.DEBUG)
      self.logger.debug("Verbose mode enabled.")


def checkHtpasswd(config):
  """XXX:set for backward compatiblity
  create a htpassword if etc/.users exist"""
  user = os.path.join(config['etc_dir'], '.users')
  htpasswdfile = os.path.join(config['etc_dir'], '.htpasswd')
  if os.path.exists(user) and not os.path.exists(htpasswdfile):
    data = open(user).read().strip().split(';')
    htpasswd = HtpasswdFile(htpasswdfile, create=True)
    htpasswd.update(data[0], data[1])
    htpasswd.save()
  else:
    return

def checkJSONConfig(config):
  """create a default json file with some parameters inside
  if the file has never been created"""
  json_file = os.path.join(config['etc_dir'], 'config.json')
  if not os.path.exists(json_file):
    params = {
      'run_instance' : True,
      'run_software' : True,
      'max_run_instance' : 3,
      'max_run_software' : 2
    }
    open(json_file, "w").write(json.dumps(params))


def run():
  "Run default configuration."
  # Parse arguments
  config = Config()
  config.setConfig()

  if os.getuid() == 0:
    # avoid mistakes (mainly in development mode)
    raise Exception('Do not run SlapRunner as root.')

  serve(config)

def serve(config):
  from werkzeug.contrib.fixers import ProxyFix
  workdir = os.path.join(config.runner_workdir, 'project')
  software_link = os.path.join(config.runner_workdir, 'softwareLink')
  app.config.update(**config.__dict__)
  app.config.update(
    software_log=config.software_root.rstrip('/') + '.log',
    instance_log=config.instance_root.rstrip('/') + '.log',
    workspace=workdir,
    software_link=software_link,
    instance_profile='instance.cfg',
    software_profile='software.cfg',
    SECRET_KEY=os.urandom(24),
    PERMANENT_SESSION_LIFETIME=datetime.timedelta(days=31),
  )
  checkHtpasswd(app.config)
  checkJSONConfig(app.config)
  if not os.path.exists(workdir):
    os.mkdir(workdir)
  if not os.path.exists(software_link):
    os.mkdir(software_link)
  setHandler()
  app.logger.addHandler(config.logger)
  repo_url = app.config['default_repository']
  branch_name = app.config.get('default_repository_branch', '')
  repo_name = os.path.basename(repo_url.replace('.git', ''))
  try:
    repository_path = os.path.join(workdir, repo_name)
    app.config.update(default_repository_path=repository_path)
    if len(os.listdir(workdir)) == 0 or not os.path.exists(repository_path):
      app.logger.info('cloning repository %s...' % repo_url)
      result = cloneRepo(repo_url, repository_path)
      if branch_name:
        switchBranch(repository_path, branch_name)
  except GitCommandError, e:
    app.logger.warning('Error while cloning default repository: %s' % str(e))
    traceback.print_exc()
  app.logger.info('Running slapgrid...')
  if app.config['auto_deploy_instance'] in TRUE_VALUES:
    import thread
    # XXX-Nicolas: Hack to be sure that supervisord has started
    # before any communication with it, so that gunicorn doesn't exit
    thread.start_new_thread(waitForRun, (app.config,))
  config.logger.info('Done.')
  app.wsgi_app = ProxyFix(app.wsgi_app)


def waitForRun(config):
    time.sleep(3)
    runInstanceWithLock(config)


def getUpdatedParameter(self, var):
  configuration_parser = ConfigParser.SafeConfigParser()
  configuration_file_path = os.path.abspath(os.getenv('RUNNER_CONFIG'))
  configuration_parser.read(configuration_file_path)

  for section in configuration_parser.sections():
    if configuration_parser.has_option(section, var):
      return configuration_parser.get(section, var)
  # if the requested var is dependant of flask
  if var in self.keys():
    temp_dict = dict()
    temp_dict.update(self)
    return temp_dict[var]
  else:
    raise KeyError

if __name__ == 'slapos.runner.run':
  flask.config.Config.__getitem__ = getUpdatedParameter
  run()



import collections
import datetime
import feedparser
import json
import os
import shutil
import tempfile
import time
import unittest

from slapos.generatefeed import generateFeed

class Option(dict):
    def __init__(self, **kw): 
        self.__dict__.update(kw)
    def __setitem__(i, y):
        self.__dict__[i] = y

class TestGenerateFeed(unittest.TestCase):
  def setUp(self):
    self.item_directory = tempfile.mkdtemp(dir='.')
    self.feed_path = os.path.join(self.item_directory, 'path')

  def tearDown(self):
    shutil.rmtree(self.item_directory)

  def getOptionObject(self, **kw):
    """
    Returns an object containing options as properties, to simulate a call
    to the tested script
    """
    option = {
      'output': self.feed_path,
      'status_item_path': self.item_directory,
      'max_item': 50,
      'feed_title': 'Feed title',
      'feed_link': 'http://example.com',
      'feed_description': 'Feed description',
    }
    option.update(kw)
    return Option(**option)

  def saveAsStatusItem(self, filename, content):
    """
    Save a JSON at filename in self.item_directory as a status item
    """
    path = os.path.join(self.item_directory, filename)
    with open(path, 'w') as status_file:
      status_file.write(json.dumps(content))

  def createItemSample(self):
    """
    Populate item_directory with a few sample items
    """
    item = [
      # Last in alphabet, first in pubDate
      ('zzz.item',
        {'description': 'description is OK too',
         'link': "http://example.com",
         'pubDate': time.mktime(datetime.datetime(2000, 1, 1).timetuple()),
         'title': 'everything is OK',
        }),
      # First in pubDate, last in alphabet 
      ('aaa.item',
        {'description': 'what went wrong ?',
         'link': "http://example.com",
         'pubDate': time.mktime(datetime.datetime(2000, 12, 31).timetuple()),
         'title': 'I guess we have an ERROR',
        }),
    ]
    for filename, content in item:
      self.saveAsStatusItem(filename, content)

  def test_feedItemsAreSortedByDate(self):
    self.createItemSample()
    option = self.getOptionObject()
    content_feed = generateFeed(option)
    feed = feedparser.parse(content_feed)

    self.assertFalse(feed.bozo)

    start_date = None
    for item in feed.entries:
      if start_date is None:
        start_date = item.published_parsed
      self.assertLessEqual(start_date, item.published_parsed)

  def test_generateFeedCleanStatusDirectoryIfTooManyItems(self):
    option = self.getOptionObject()
    option.max_item = 3

    # Creates items more than allowed
    item_dummy_content = {
      'description': 'dummy description',
      'link': "http://example.com",
      'pubDate': time.mktime(datetime.datetime.now().timetuple()),
      'title': 'dummy title',
    }
    for i in range(5):
      filename = '%s.item' % i
      self.saveAsStatusItem(filename, item_dummy_content)
      time.sleep(1)
      item_dummy_content['pubDate'] = time.mktime(datetime.datetime.now().timetuple())

    content_feed = generateFeed(option)
    feed = feedparser.parse(content_feed)

    self.assertFalse(feed.bozo)

    # Feed entries number should be limited
    self.assertEqual(len(feed.entries), option.max_item)

    # Status item directory should have been cleaned
    self.assertEqual(len(os.listdir(self.item_directory)), option.max_item)

    # Only "younger" items should still be there
    remaining_status_item_list = os.listdir(self.item_directory)
    expected_remaining_item_list = []

    for i in range(5-3, 5): # older items (from 1 to 2) have been deleted
      expected_remaining_item_list.append('%s.item' % i)

    self.assertItemsEqual(remaining_status_item_list,
                          expected_remaining_item_list)

if __name__ == '__main__':
  unittest.main()



import argparse
import re
import time
import sys
from slapos.networkbench.ping import ping, ping6

def test(address, ipv4, count):

  if ipv4:
    return ping(address, count=count)

  return ping6(address, count=count)

def main():
  parser = argparse.ArgumentParser()
  # Address to ping to
  parser.add_argument("-a", "--address", required=True)
  # Force use ipv4 protocol
  parser.add_argument("-4", "--ipv4", action="store_true" )
  parser.add_argument("-c", "--count", metavar="COUNT", default=10 )
  args = parser.parse_args()

  result = test(args.address, args.ipv4, args.count)

  print "%s host=%s code=%s, result=%s, packet_lost_ratio=%s msg=%s" % result
  if result[4] != "0":
    # Packet lost occurred
    print "FAIL"
    sys.exit(1)
  print "OK"




import subprocess
import re

# rtt min/avg/max/mdev = 1.102/1.493/2.203/0.438 ms
ping_re = re.compile(
    ".*"
    "(?P<min>[\d\.]+)/"
    "(?P<avg>[\d\.]+)/"
    "(?P<max>[\d\.]+)/"
    "(?P<mdev>[\d\.]+) ms"
    )

date_reg_exp = re.compile('\d{4}[-/]\d{2}[-/]\d{2}')


def ping(host, timeout=10, protocol="4", count=10):
  if protocol == '4':
    ping_bin = 'ping'
    test_title = 'PING'
  elif protocol == '6':
    ping_bin = 'ping6'
    test_title = 'PING6'

  proc = subprocess.Popen((ping_bin, '-c', str(count), '-w', str(timeout), host),
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)

  out, err = proc.communicate()
  if 'Network is unreachable' in err:
    return (test_title, host, 600, 'failed', 100, "Network is unreachable")
  try:
    packet_loss_line, summary_line = (out.splitlines() or [''])[-2:]
  except:
    return (test_title, host, 600, 'failed', -1, "Fail to parser ping output")
  m = ping_re.match(summary_line)
  match = re.search('(\d*)% packet loss', packet_loss_line)
  packet_lost_ratio = match.group(1)

  info_list = (test_title, host, 600, 'failed', packet_lost_ratio, "Cannot ping host")
  if packet_lost_ratio != 0:
    if m:
      info_list = (test_title, host, 200, m.group('avg'), packet_lost_ratio,
           'min %(min)s max %(max)s avg %(avg)s' % m.groupdict())
  else:
    info_list = (test_title, host, 600, 'failed', packet_lost_ratio,
      "You have package Lost")

  return info_list

def ping6(host, timeout=10, count=10):
  return ping(host, timeout=10, protocol='6', count=count)




import argparse
import re
import time
import sys
from slapos.networkbench.ping import ping, ping6

def test(ipv6, ipv4, count):

  result_ipv4 = ping(ipv4, count=count)
  print "%s host=%s code=%s, result=%s, packet_lost_ratio=%s msg=%s" % result_ipv4

  result_ipv6 = ping6(ipv6, count=count)
  print "%s host=%s code=%s, result=%s, packet_lost_ratio=%s msg=%s" % result_ipv6

  if result_ipv4[3] == "failed" and result_ipv6[3] != "failed":
    # IPv4 is unreacheable
    return "OK"

  if result_ipv6[3] == "failed":
    # IPv6 is unreacheable
    return "FAILED"

  latency4 = float(result_ipv4[3])
  latency6 = float(result_ipv6[3])
  # We can consider that at worst 1ms is added to
  # ipv4 response, due the usage of openvpn.
  acceptable_delay = 1

  # We can consider that we accept a certain increase
  # on latency, if we are on a bit congested link.
  # So 10% is reseonable enough.
  acceptable_lost = 0.10

  # Increase latency with the value.
  latency4 += acceptable_delay + latency4*acceptable_lost
  if latency4 < latency6:
    print "Fail %s (latency4) > %s (latence6)" % (latency4, latency6)
    return "FAIL"

  # Compare if both has Same working rate
  return "OK"

def main():
  parser = argparse.ArgumentParser()
  # promise ipv6 and ipv4 address to compare.
  parser.add_argument("-4", "--ipv4", required=1 )
  parser.add_argument("-6", "--ipv6", required=1)

  parser.add_argument("-c", "--count", default=10 )
  args = parser.parse_args()

  result = test(args.ipv6, args.ipv4, args.count)

  print result
  if result != "OK":
    # re6st is not on an optimal state.
    sys.exit(1)




import socket
import time
import dns.resolver

def resolve(name, expected_list=None):
  """ Resolve name using standard system name resolution.
  """
  begin = time.time()
  try:
    ip_list = [i.to_text() for i in dns.resolver.query(name, "A")]
    resolution = 200
    status = "OK"
  except dns.resolver.NXDOMAIN:
    resolution = 600
    status = "Cannot resolve the hostname"
    ip_list = []

  resolving_time = time.time() - begin
  # Out put is:
  # TEST IDENTIFIER, NAME, RESOLUTION (200 or 600), Time for resolve,
  # status ("OK" or "Cannot resolve the hostname"), Resolved IP.
  if expected_list is not None and set(expected_list) != set(ip_list):
    status = "UNEXPECTED"
    ip_list = "%s (expected) != %s (found)" % (expected_list, ip_list)
  return ('DNS', name, resolution, resolving_time, status, ip_list)


